// A car with normal wheels (Created by .:..: 8.1.2008)
Class WheeledCarPhys extends Vehicle;

Enum WheelTypeType
{
	TT_None,
	TT_TurningWheel,
	TT_RevesedTurn
};
struct WheelsTypes
{
	var() vector WheelOffset;
	var() rotator WheelRot;
	var() class<VehicleWheel> WheelClass;
	var() mesh WheelMesh;
	var() WheelTypeType WheelType;
};
var() WheelsTypes Wheels[8];
var() float MaxGroundSpeed,WheelMaxYaw,WheelTurnSpeed;
var VehicleWheel MyWheels[8];
var byte NumWheels,WheelsTurning;
var bool bHasWheelMeshes,bWasStuckOnW,bReversing;
var float WheelYaw,StuckTimer,ReverseTimer;
var int WheelsPitch;

replication
{
	// Variables the server should send to the client.
	unreliable if( Role==ROLE_Authority && !bNetOwner )
		WheelsTurning;
}

simulated function ClientUpdateState( float Delta )
{
	Super.ClientUpdateState(Delta);
	if( WheelsTurning>0 )
	{
		Turning = int(WheelsTurning)-2;
		WheelsTurning = 0;
	}
}
function ServerPackState( float Delta)
{
	Super.ServerPackState(Delta);
	WheelsTurning = Turning+2;
}
simulated function PostBeginPlay()
{
	local byte i;

	Super.PostBeginPlay();
	if( Level.NetMode!=NM_DedicatedServer )
	{
		For( i=0; i<ArrayCount(Wheels); i++ )
		{
			if( Wheels[i].WheelClass==None )
				Continue;
			MyWheels[NumWheels] = VehicleWheel(AddAttachment(Wheels[i].WheelClass));
			if( MyWheels[NumWheels]==None )
				Continue;
			MyWheels[NumWheels].WheelOffset = Wheels[i].WheelOffset;
			MyWheels[NumWheels].WheelRot = Wheels[i].WheelRot;
			MyWheels[NumWheels].TurnType = Wheels[i].WheelType;
			MyWheels[NumWheels].Mesh = Wheels[i].WheelMesh;
			NumWheels++;
			if( !bHasWheelMeshes )
				bHasWheelMeshes = True;
		}
	}
}
// Return normal for acceleration direction.
simulated function vector GetAccelDir( int InTurn, int InRise, int InAccel )
{
	local rotator R;

	R.Yaw = VehicleYaw+WheelYaw/2;
	Return SetUpNewMVelocity(vector(R)*InAccel,ActualFloorNormal,0);
}

simulated function UpdateDriverInput( float Delta )
{
	local vector Ac,NVeloc;
	local float DesTurn,DeAcc,DeAccRat;
	local rotator R;

	R = TransformForGroundRot(VehicleYaw,FloorNormal);
	if( Rotation!=R )
		SetRotation(R);
	DesTurn = WheelMaxYaw*Turning*-1;
	if( WheelYaw!=DesTurn )
	{
		if( WheelYaw<DesTurn )
		{
			WheelYaw+=WheelTurnSpeed*Delta;
			if( WheelYaw>DesTurn )
				WheelYaw = DesTurn;
		}
		else
		{
			WheelYaw-=WheelTurnSpeed*Delta;
			if( WheelYaw<DesTurn )
				WheelYaw = DesTurn;
		}
	}
	if( !bOnGround )
	{
		Velocity+=Region.Zone.ZoneGravity*Delta*VehicleGravityScale;
		Return;
	}
	if( Level.NetMode==NM_Client && !IsNetOwner(Owner) )
		Return;

	if( bOnGround )
		Velocity+=CalcGravityStrength(Region.Zone.ZoneGravity*VehicleGravityScale,FloorNormal)*Delta/(Region.Zone.ZoneGroundFriction/8.f+1.f);
	DesTurn = VSize(Velocity)*WheelYaw*Delta/400*GetMovementDir();
	VehicleYaw+=DesTurn;
	if( !bCameraOnBehindView && Driver!=None )
		Driver.ViewRotation.Yaw+=DesTurn;

	// Update vehicle speed
	if( Accel==0 )
	{
		DeAcc = VSize(Velocity);
		DeAccRat = Delta*WAccelRate*Region.Zone.ZoneGroundFriction;
		if( DeAccRat>DeAcc )
			DeAccRat = DeAcc;
		Velocity-=Normal(Velocity)*DeAccRat;
		Return;
	}
	DeAcc = VSize(Velocity);
	if( DeAcc<MaxGroundSpeed )
	{
		DeAcc+=WAccelRate*Delta;
		if( DeAcc>MaxGroundSpeed )
			DeAcc = MaxGroundSpeed;
	}
	else DeAcc+=WAccelRate*Delta/100;
	Ac = GetAccelDir(Turning,Rising,Accel);
	NVeloc = Normal(Velocity);
	if( DeAcc>50 && (Ac Dot NVeloc)<0.4 )
	{
		Velocity+=Ac*Region.Zone.ZoneGroundFriction*WAccelRate*Delta*2.f;
		Return;
	}
	Ac = Ac*MaxGroundSpeed*Region.Zone.ZoneGroundFriction/10;
	Velocity = Normal(Velocity+Ac)*DeAcc;
}
// 1 - Forward
// -1 - Reversed
simulated function int GetMovementDir()
{
	if( (Normal(Velocity) dot vector(Rotation))>0 )
		Return 1;
	else Return -1;
}
function int ShouldAccelFor( vector AcTarget )
{
	local bool bStuck;

	if( bReversing )
	{
		if( ReverseTimer<Level.TimeSeconds )
		{
			bReversing = False;
			StuckTimer = Level.TimeSeconds+3;
			Return 1;
		}
		Return -1;
	}
	bStuck = VSize(Velocity)<MaxGroundSpeed/5;
	if( bWasStuckOnW!=bStuck )
	{
		bWasStuckOnW = bStuck;
		if( bStuck )
			StuckTimer = Level.TimeSeconds+2;
	}
	if( bStuck && StuckTimer<Level.TimeSeconds )
	{
		bReversing = True;
		ReverseTimer = Level.TimeSeconds+2;
		Return -1;
	}
	Return 1;
}
function int ShouldTurnFor( vector AcTarget )
{
	if( bReversing )
		Return Super.ShouldTurnFor(AcTarget)*-1;
	else Return Super.ShouldTurnFor(AcTarget);
}
simulated function AttachmentsTick( float Delta )
{
	local byte i,bSet[3];
	local rotator R,SR[3];
	local Quat WQ,VehQ;

	// Update wheels turning
	if( bHasWheelMeshes )
	{
		VehQ = RtoQ(Rotation);
		WheelsPitch-=GetMovementDir()*VSize(Velocity)*200.f*Delta;
		While( WheelsPitch>UU_360_DEGREES )
			WheelsPitch-=UU_360_DEGREES;
		While( WheelsPitch>UU_360_NEGDEGREES )
			WheelsPitch-=UU_360_NEGDEGREES;
		For( i=0; i<NumWheels; i++ )
		{
			MyWheels[i].SetLocation(Location+(MyWheels[i].WheelOffset >> Rotation)*DrawScale);
			if( bSet[MyWheels[i].TurnType]==0 )
			{
				bSet[MyWheels[i].TurnType] = 1;
				R = MyWheels[i].WheelRot;
				if( MyWheels[i].TurnType==1 )
					R.Yaw+=WheelYaw;
				else if( MyWheels[i].TurnType==2 )
					R.Yaw-=WheelYaw;
				R.Pitch = WheelsPitch;
				WQ = RtoQ(R);
				WQ = WQ Qmulti VehQ;
				SR[MyWheels[i].TurnType] = QtoR(WQ);
			}
			MyWheels[i].SetRotation(SR[MyWheels[i].TurnType]);
		}
	}
}

defaultproperties
{
				Wheels(0)=(WheelOffset=(X=25.320000,Y=28.540001,Z=-23.000000),WheelClass=Class'CSR.VehicleWheel',WheelMesh=LodMesh'UnrealShare.WoodenBoxM',WheelType=TT_TurningWheel)
				Wheels(1)=(WheelOffset=(X=25.320000,Y=-28.540001,Z=-23.000000),WheelClass=Class'CSR.VehicleWheel',WheelMesh=LodMesh'UnrealShare.WoodenBoxM',WheelType=TT_TurningWheel)
				Wheels(2)=(WheelOffset=(X=-25.320000,Y=28.540001,Z=-23.000000),WheelClass=Class'CSR.VehicleWheel',WheelMesh=LodMesh'UnrealShare.WoodenBoxM')
				Wheels(3)=(WheelOffset=(X=-25.320000,Y=-28.540001,Z=-23.000000),WheelClass=Class'CSR.VehicleWheel',WheelMesh=LodMesh'UnrealShare.WoodenBoxM')
				MaxGroundSpeed=3000.000000
				WheelMaxYaw=8000.000000
				WheelTurnSpeed=12000.000000
				WAccelRate=500.000000
				Health=500
				bFPViewUseRelRot=True
				bFPRepYawUpdatesView=True
				VehicleName="Wheeled Car"
				TranslatorDescription="This is a wheeled vehicle, press [Fire] or [AltFire] to fire the different firemodes. Use your Strafe keys to turn this vehicle and Move Forward/Backward keys to accelerate/deaccelerate. To leave this vehicle, press your [ThrowWeapon] key."
				VehicleKeyInfoStr="Wheeled car keys:|%MoveForward%,%MoveBackward% to accelerate/deaccelerate|%StrafeLeft%, %StrafeRight% to turn|%Fire% to fire, %AltFire% to alt fire|Number keys to switch seats|%ThrowWeapon% to exit the vehicle"
				Mesh=LodMesh'UnrealShare.WoodenBoxM'
}
